<?php

require_once('../../config/config.inc.php');
require_once('./okazjeintegrator.php');
set_time_limit(240);
error_reporting(-1);
ini_set("memory_limit", "100M");
header("Content-type: text/xml");

/**
 * Klasa do generowania pliku XML
 */
class OkazjeIntegratorXml extends OkazjeIntegrator
{
    /**
     * @var DOMDocument
     */
    private $xml;

    public function __construct()
    {
        $this->xml = new DOMDocument();
    }

    /**
     * Akcja generowanie pliku XML dla okazje.info
     *
     * @return DOMDocument
     */
    protected function generateXML()
    {
        $link = new Link();
        $this->xml = new DOMDocument();

        $shop_size = explode(';', Configuration::get('OKAZJEINTEGRATOR_SHOP_SIZE'));
        $shop_color = explode(';', Configuration::get('OKAZJEINTEGRATOR_SHOP_COLOR'));

        if (is_array($shop_size) && !empty($shop_size)) {
            $shop_size = array_map('trim', $shop_size);
            $shop_size = array_map('mb_strtolower', $shop_size);
        }

        if (is_array($shop_color) && !empty($shop_color)) {
            $shop_color = array_map('trim', $shop_color);
            $shop_color = array_map('mb_strtolower', $shop_color);
        }

        $okazje_size = self::getOkazjeSize();
        $okazje_color = self::getOkazjeColor();

        $Products = $this->getProducts();
        $Categories = $this->_getCategoryTree();

        if ($Products && $Categories) {
            $this->xml->loadXML('<?xml version="1.0" encoding="UTF-8" ?><okazje><offers/></okazje>');

            $offers = $this->xml->getElementsByTagName('offers');
            $offers = $offers->item(0);

            foreach ($Products AS $Product) {
                $attributes = self::getAttributesForProduct($Product['id_product']);

                $offer = $this->xml->createElement('offer');
                $offers->appendChild($offer);

                // id
                $this->addElement($offer, 'id', $Product['id_product']);

                // name
                $this->addElement($offer, 'name', self::htmlentities($Product['name']));

                // description
                $this->addElement($offer, 'description', strip_tags(html_entity_decode($Product['description'], ENT_COMPAT, "UTF-8"), "<br><li><p><ul><tr>"), true);

                // reference
                if ($Product['reference'] != '') {
                    $this->addAttribute($offer, 'code', $Product['reference'], true);
                }

                // ean
                if ($Product['ean13'] != ''  && $Product['ean13'] != '0') {
                    $this->addAttribute($offer, 'ean', $Product['ean13'], true);
                }

                // available
                if ((int) $Product['available_for_order'] > 0) {
                    $this->addElement($offer, 'available', $Product['available_for_order']);
                } else {
                    $this->addElement($offer, 'available', $Product['available_now']);
                }

                // attributes
                if (is_array($attributes) && !empty($attributes)) {
                    foreach ($attributes as $a) {
                        $explode = false;

                        if (in_array(trim(mb_strtolower($a['group_name'])), $shop_size)) {
                            $value = $okazje_size;
                        } elseif (in_array(trim(mb_strtolower($a['group_name'])), $shop_color) || $a['is_color_group'] == 't') {
                            $value = $okazje_color;
                            $explode = true;
                        } elseif (preg_match('#(rozmiar|size)+#', mb_strtolower(trim($a['group_name'])))) {
                            $value = $okazje_size;
                        } else {
                            $explode = true;
                            $value = $a['group_name'];
                        }

                        if ($explode) {
                            $attribute_names = explode(';', $a['attribute_name']);

                            if (is_array($attribute_names) && !empty($attribute_names)) {
                                foreach ($attribute_names as $attribute_name) {
                                    $this->addAttribute($offer, $value, $attribute_name);
                                }
                            }
                        } else {
                            $this->addAttribute($offer, $value, $a['attribute_name']);
                        }
                    }
                }

                // category
                $this->addElement($offer, 'category', $Categories[$Product['id_category_default']]['path']);

                // price
                $this->addElement($offer, 'price', Product::getPriceStatic($Product['id_product']));
                if (Product::isDiscounted($Product['id_product'])) {
                    $this->addElement($offer, 'old_price', Product::getPriceStatic($Product['id_product'], true, NULL, 2, NULL, false, false));
                }

                // url
                $this->addElement($offer, 'url', htmlspecialchars($link->getProductLink($Product['id_product'], $Product['link_rewrite'])));

                // producer
                $this->addElement($offer, 'producer', $Product['manufacturer_name']);

                // image
                $image = Image::getCover($Product['id_product']);
                if ($image['id_image']) {
                    $img_url = $this->getImageLink($Product['link_rewrite'], $image['id_product'] . '-' . $image['id_image'], $this->getXmlImageType());
                } else {
                    $img_url = "";
                }
                $this->addElement($offer, 'image', $img_url);
            }
        }

        return $this->xml;
    }

    /**
     * Add child element to XML object
     *
     * @param DOMElement $element
     * @param string $name
     * @param string $value
     * @param bool $cdata
     */
    private function addElement($element, $name, $value = '', $cdata = false)
    {
        if ($cdata) {
            $data = $this->xml->createCDATASection($value);
            $obj = $this->xml->createElement($name);
            $obj->appendChild($data);
        } else {
            $obj = $this->xml->createElement($name, $value);
        }

        $element->appendChild($obj);
    }

    /**
     * Add attribute child
     *
     * @param DOMElement $element
     * @param string $name
     * @param string $value
     * @param bool $cdata
     */
    private function addAttribute($element, $name, $value = '', $cdata = false)
    {
        if ($cdata) {
            $data = $this->xml->createCDATASection($value);
            $attribute = $this->xml->createElement('attribute');
            $attribute->appendChild($data);
        } else {
            $attribute = $this->xml->createElement('attribute', $value);
        }

        $nameAttr = $this->xml->createAttribute('name');
        $nameAttr->value = $name;
        $attribute->appendChild($nameAttr);

        $element->appendChild($attribute);
    }
}

if (class_exists('Context', false)) {
    $fc = new FrontController;
    $fc->init();
}

$OkazjeIntegrator = new OkazjeIntegratorXml();
$OkazjeIntegrator->getXml();
