<?php

/**
 * Moduł okazjeintegrator do prestashop
 */
class OkazjeIntegrator extends Module
{
    private $_html = '';
    private $_postErrors = array();
    public static $cache_getShopHost = NULL;
    public $okazje_color = 'Kolor';
    public $okazje_size = 'Rozmiar';
    public $debug_cache = null;

    function __construct()
    {
        global $protocol_content;

        if (empty($protocol_content)) {
            $protocol_content = 'http://';
        }

        $this->name = 'okazjeintegrator';

        if ($this->isNewPS()) {
            $this->tab = 'advertising_marketing';
        } else {
            $this->tab = 'Advertisement';
        }

        $this->author = 'Okazje.info';
        $this->version = '1.2.1';

        parent::__construct();

        if (!defined("_PS_BASE_URL_"))
            define('_PS_BASE_URL_', $this->getShopHost(false));

        /* The parent construct is required for translations */
        $this->page = basename(__FILE__, '.php');
        $this->displayName = $this->l('Okazje Integrator');
        $this->description = $this->l('Generuje xml z ofertą sklepu okazje.info');
    }

    public function getOkazjeSize()
    {
        return $this->okazje_size;
    }

    public function getOkazjeColor()
    {
        return $this->okazje_color;
    }

    /**
     * Get default color attribute
     *
     * @return false|null|string
     */
    private function getDefaultColorAttribute()
    {
        $color_attibute_name = Db::getInstance()->getValue("
			SELECT name 
			FROM " . _DB_PREFIX_ . "attribute_group_lang
			JOIN " . _DB_PREFIX_ . "attribute_group USING (id_attribute_group) 
			WHERE is_color_group IS TRUE");
        return $color_attibute_name ? $color_attibute_name : $this->okazje_color;
    }

    /**
     * Install action
     *
     * @return bool
     */
    public function install()
    {
        $default_xml_image_size = Db::getInstance()->getValue("SELECT name, width, height FROM `" . _DB_PREFIX_ . "image_type` WHERE `products` = 1 ORDER BY height DESC LIMIT 1");
        $default_xml_image_size = $default_xml_image_size ? $default_xml_image_size : 'medium';

        if (
            !parent::install()
            OR !Configuration::updateValue('OKAZJEINTEGRATOR_EXP_WITH_ZERO', '0')
            OR !Configuration::updateValue('OKAZJEINTEGRATOR_REDUCE_AVAIL', '0')
            OR !Configuration::updateValue('OKAZJEINTEGRATOR_DEBUG', '0')
            OR !Configuration::updateValue('OKAZJEINTEGRATOR_SHOP_COLOR', $this->getDefaultColorAttribute())
            OR !Configuration::updateValue('OKAZJEINTEGRATOR_SHOP_SIZE', $this->okazje_size)
            OR !Configuration::updateValue(strtoupper($this->name) . '_XML_IMAGE_TYPE', $default_xml_image_size)
            OR !$this->registerHook('backOfficeHome')
        ) {
            return false;
        }

        if (
            $this->isNewPS()
            AND (
                !$this->registerHook('categoryUpdate')
                OR !$this->registerHook('categoryDeletion')
            )
        ) {
            return false;
        }

        return true;
    }

    /**
     * Uninstall action
     *
     * @return bool
     */
    public function uninstall()
    {
        if (
            !Configuration::deleteByName('OKAZJEINTEGRATOR_EXP_WITH_ZERO')
            OR !Configuration::deleteByName('OKAZJEINTEGRATOR_REDUCE_AVAIL')
            OR !Configuration::deleteByName('OKAZJEINTEGRATOR_DEBUG')
            OR !Configuration::deleteByName('OKAZJEINTEGRATOR_SHOP_COLOR')
            OR !Configuration::deleteByName('OKAZJEINTEGRATOR_SHOP_SIZE')
            OR !Configuration::deleteByName(strtoupper($this->name) . '_XML_IMAGE_TYPE')
            OR !parent::uninstall()
        ) {
            return false;
        }

        return true;
    }

    /**
     * Get "pl" language id
     *
     * @return mixed
     */
    protected function getLangId()
    {
        return (Language::getIdByIso("pl") ? Language::getIdByIso("pl") : Configuration::get('PS_LANG_DEFAULT'));
    }

    /**
     * Get shop host
     *
     * @param bool $with_dir
     *
     * @return string
     */
    protected function getShopHost($with_dir = true)
    {

        if (self::$cache_getShopHost === NULL) {
            if (strpos(_PS_VERSION_, '1.2') === 0) {
                self::$cache_getShopHost = 'http://' . $_SERVER['HTTP_HOST'];

            } elseif (strpos(_PS_VERSION_, '1.3') === 0) {
                self::$cache_getShopHost = Tools::getHttpHost(true);

            } else {
                self::$cache_getShopHost = Tools::getShopDomainSsl(true);
            }
        }

        return self::$cache_getShopHost . ($with_dir ? __PS_BASE_URI__ : NULL);
    }

    /**
     * Check if Prestashop version > 1.4
     *
     * @return bool
     */
    public function isNewPS()
    {
        return version_compare(_PS_VERSION_, '1.4') >= 0 ? true : false;
    }

    /**
     * Get image link - Extends for prestashop method
     *
     * @param $name
     * @param $ids
     * @param null $type
     *
     * @return string
     */
    public function getImageLink($name, $ids, $type = null)
    {
        if (class_exists('Context', false)) {
            $link = new Link(null, 'http://');
        } else {
            $link = new Link();
        }

        $url = $link->getImageLink($name, $ids, $type);

        if (strpos($url, 'http') !== 0) {
            return (rtrim($this->getShopHost(false), '/') . $url);
        }

        return $url;
    }

    /**
     * Module view in admin panel
     *
     * @return string
     */
    public function getContent()
    {
        $this->current_version = $this->getVersion();

        $this->_html = null;
        $this->_postValidateProcess();
        $this->_html .=
            "<div style='border: 2px solid #666; padding: 5px;height: 65px;'>
			<img style='margin: 5px 15px 5px 10px; float: left;' src='http://www.okazje.info.pl/r/i/2/logo-main.png' alt='' />
			</div><br><h2>Okazje.info</h2>";

        if ($this->version != $this->current_version) {
            $this->_html .= '<div style="color:red;font-weight:bold">Dostępna jest nowa wersja modułu Okazje Integrator v' . $this->current_version . '</div><br>';
        }

        $this->_html .= $this->_displaySettingsForm();
        $this->_html .=
            '<br/>
			<fieldset class="width4">
			<legend>
			<img src="../img/admin/warning.gif" />Informacje</legend>
			Przy nawiązywaniu współpracy z porównywarką Okazje.info podaj link z lokalizacja pliku xml zawierającego oferty sklepu:<br /><br />
			<ul class="pw_links_list">
			<li><b>okazje.info.pl:</b>
			<input style="width:600px" readonly="readonly" value="' . $this->getShopHost() . 'modules/' . $this->name . '/offer.php"/></li>
			</ul>
			</fieldset>';

        return $this->_html;
    }

    /**
     * Module form view in admin panel
     *
     * @return null|string
     */
    protected function _displaySettingsForm()
    {
        $return = null;

        $exportwithzero = intval(Configuration::get('OKAZJEINTEGRATOR_EXP_WITH_ZERO'));
        $reduceavailibilty = intval(Configuration::get('OKAZJEINTEGRATOR_REDUCE_AVAIL'));
        $xmlImageType = Configuration::get(strtoupper($this->name) . '_XML_IMAGE_TYPE');
        $color = Configuration::get('OKAZJEINTEGRATOR_SHOP_COLOR');
        $size = Configuration::get('OKAZJEINTEGRATOR_SHOP_SIZE');
        $debug = (int)Configuration::get('OKAZJEINTEGRATOR_DEBUG');

        if (sizeof($this->_postErrors)) {
            $this->_displayErrors();
        }

        $return .= '
		<form action="' . $_SERVER['REQUEST_URI'] . '" method="post">
		<fieldset class="width4">
			<legend><img src="../img/admin/contact.gif" />' . $this->l('Ustawienia') . '</legend>';

        $return .= '<label>' . $this->l('Eksportuj z zerowym stanem magazynowym') . '</label>
			<div class="margin-form">
				Eksportuje produkty, które posiadają stan magazynowy 0. Gdy nie prowadzisz stanów magazynowych.<br>
				<input type="radio" ' . ($exportwithzero === 0 ? ' checked="checked" ' : '') . ' name="exportwithzero" value="0" /><b>(domyślne)</b> Nie eksporuj<br>
				<input type="radio" ' . ($exportwithzero === 1 ? ' checked="checked" ' : '') . ' name="exportwithzero" value="1" /> Eksportuj (jako dostępne; stan wymuszony na 1) (Tip: Zaznacz, gdy masz na sklepie atrybuty (np. kolory), a główny produkt ma stan magazynowy zero )<br>
				<input type="radio" ' . ($exportwithzero === 2 ? ' checked="checked" ' : '') . ' name="exportwithzero" value="2" /> Eksportuj (jako niedostępne; stan wszystkich produktów wymuszony na 0)<br>
				
			</div>
			<br>
			<label>' . $this->l('Redukuj dostępność') . '</label>
			<div class="margin-form">
				<input type="radio" ' . ($reduceavailibilty === 0 ? ' checked="checked" ' : '') . ' name="reduceavailibilty" value="0" /><b>(domyślne)</b> Eksportuj produkty z prawdziwym stanem magazynowym; Produkty wymuszone przez poprzenią opcję będą eksportowane ze stanem magazynowym ustalonym wyżej. <br>
				<input type="radio" ' . ($reduceavailibilty === 1 ? ' checked="checked" ' : '') . ' name="reduceavailibilty" value="1" /> Redukuj stany większe niż 1 na 1; zero pozostaje zerem.
			</div>
			
			<br>
			<label>' . $this->l('Debugowanie') . '</label>
			<div class="margin-form">
				<input type="radio" ' . ($debug === 0 ? ' checked="checked" ' : '') . ' name="pwdebug" value="0" />Nie<br>
				<input type="radio" ' . ($debug === 1 ? ' checked="checked" ' : '') . ' name="pwdebug" value="1" />Tak
				<br>
				Tryb debugowania ułatwia stwierdzenie, z jakiego powodu produkty się nie wyświetlają.
				Gdy opcja jest włączona, zamiast XML będzie generowany log z informacjami co się dzieje.
				Aby sprawdzić dlaczego produkty się nie wyświetlają, proszę włączyć tę opcję a następnie otworzyć link do generowania XML.
			</div>

			<br>
			<label>' . $this->l('Rozmiar') . '</label>
			<div class="margin-form">
				<input type="text" value="' . ($size ? $size : $this->okazje_size) . '" name="rozmiar" /><br>
				Podaj nazwę atrybutu "rozmiar". Kilka atrybutów rozdziel znakiem średnika <br>';

        $return .= '</div><br>
			<label>' . $this->l('Kolor') . '</label>
			<div class="margin-form">
				<input type="text" value="' . ($color ? $color : $this->okazje_color) . '" name="kolor" /><br>
				Podaj nazwę atrybutu "kolor". Kilka atrybutów rozdziel znakiem średnika <br>';

        $return .= '</div><br>
			<label>' . $this->l('Wybierz jaki typ obrazka ma być udostępniany w xml') . '</label>
			<div class="margin-form">
			<select name="xmlimagetype">
			';

        $images = db::getInstance()->ExecuteS("SELECT name, width, height FROM `" . _DB_PREFIX_ . "image_type` WHERE `products` = 1");

        foreach ($images as $image) {
            $return .= '<option ' . ($xmlImageType === $image['name'] ? "selected=\"selected\"" : NULL) . ' value="' . $image['name'] . '">' . $image['name'] . ' (' . $image['width'] . 'px x ' . $image['height'] . 'px)</option>';
        }

        $return .= '</select></div>
		<br /><center><input type="submit" name="submitOkazjeIntegrator" value="' . $this->l('Zapisz zmiany') . '" class="button" /></center>
		</fieldset>

		</form>';

        return $return;
    }

    /**
     * Add debug message
     *
     * @param $msg
     */
    public function doDebug($msg)
    {
        if ((int)Configuration::get('OKAZJEINTEGRATOR_DEBUG')) {
            $this->debug_cache .= $msg . "<br />\r\n";
        }
    }

    /**
     * Display debug messages
     */
    public function showDebug()
    {
        if ((int)Configuration::get('OKAZJEINTEGRATOR_DEBUG')) {
            header("content-type: text/html");
            echo $this->debug_cache;
        }
    }

    /**
     * Prestashop module - post action
     */
    protected function _postValidateProcess()
    {
        if (isset($_POST['submitOkazjeIntegrator'])) {
            Configuration::updateValue('OKAZJEINTEGRATOR_SHOP_SIZE', $_POST['rozmiar']);
            Configuration::updateValue('OKAZJEINTEGRATOR_SHOP_COLOR', $_POST['kolor']);
            Configuration::updateValue('OKAZJEINTEGRATOR_EXP_WITH_ZERO', intval($_POST['exportwithzero']));
            Configuration::updateValue('OKAZJEINTEGRATOR_REDUCE_AVAIL', intval($_POST['reduceavailibilty']));
            Configuration::updateValue('OKAZJEINTEGRATOR_DEBUG', intval($_POST['pwdebug']));
            Configuration::updateValue(strtoupper($this->name) . '_XML_IMAGE_TYPE', $_POST['xmlimagetype']);
        }
    }

    /**
     * Prestashop module - display error
     */
    protected function _displayErrors()
    {
        $nbErrors = sizeof($this->_postErrors);
        $this->_html .= '
		<div class="alert error">
			<h3>' . ($nbErrors > 1 ? $this->l('Wystapily') : $this->l('Wystapil')) . ' ' . $nbErrors . ' ' . ($nbErrors > 1 ? $this->l('bledy') : $this->l('blad')) . '</h3>
			<ol>';
        foreach ($this->_postErrors AS $error)
            $this->_html .= '<li>' . $error . '</li>';
        $this->_html .= '
			</ol>
		</div>';
    }

    /**
     * Get category tree in array
     *
     * @param string $delimeter
     *
     * @return array
     */
    protected function _getCategoryTree($delimeter = '/')
    {

        $categories = Category::getCategories($this->getLangId(), false, false);
        $categoryArray = array();

        foreach ($categories as $category) {
            $categoryArray[$category['id_category']] = $category;
        }

        foreach ($categoryArray as $category) {
            $categoryParent = $categoryArray[$category['id_category']]['id_parent'];
            $categoryPath = $categoryArray[$category['id_category']]['name'];
            $counter = 0;
            while ($categoryParent) {
                $counter++;
                if ($counter > 100)
                    break;
                if ($categoryParent != @$categoryArray[$categoryParent]['id_parent']) {
                    if (@$categoryArray[$categoryParent]['id_category'] != 1)
                        $categoryPath = @$categoryArray[$categoryParent]['name'] . $delimeter . $categoryPath;
                    $categoryParent = @$categoryArray[$categoryParent]['id_parent'];
                } else {
                    $categoryParent = 0;
                }
            }
            $categoryArray[$category['id_category']]['path'] = $categoryPath;
        }

        return $categoryArray;
    }

    /**
     * Generate XML - THIS METHOD HAS TO BE IMPLEMENTED IN EXTENDED CLASS
     */
    protected function generateXML()
    {
        die('Problem z generowaniem XML Objekt nie posiada poprawnej klasy do generowania XML');
    }

    /**
     * Public method to get XML
     */
    public function getXML()
    {
        $xml = $this->generateXML();

        if ((int)Configuration::get('OKAZJEINTEGRATOR_DEBUG')) {
            $this->showDebug();
        } else {
            echo $xml->saveXML();
        }
    }

    /**
     * Get xml image type from config variable
     *
     * @return mixed
     */
    protected function getXmlImageType()
    {
        return Configuration::get(strtoupper($this->name) . '_XML_IMAGE_TYPE');
    }

    /**
     * Get the list of products
     *
     * @param bool $active
     * @param bool $instock
     *
     * @return mixed
     */
    protected function getProducts($active = true, $instock = true)
    {
        Global $cookie;
        $cookie = new Cookie('ps');

        $Products = Product::getProducts($this->getLangId(), 0, null, 'name', 'ASC', false, (bool)$active);

        $exportwithzero = intval(Configuration::get('OKAZJEINTEGRATOR_EXP_WITH_ZERO'));
        $reduceavailibilty = intval(Configuration::get('OKAZJEINTEGRATOR_REDUCE_AVAIL'));

        foreach ($Products as $key => &$product) {
            $p = new Product($product['id_product']);

            if ($exportwithzero === 0 AND $product['quantity'] <= 0) { // to usun

                if (method_exists('Product', 'getQuantity'))
                    $product['quantity'] = Product::getQuantity($product['id_product']);

                if ($product['quantity'] <= 0) {
                    $this->doDebug('<span style="color: grey;">' . $product['name'] . '</span> stan magazynowy = 0, usuwanie z listy.');
                    unset($Products[$key]);
                    continue;
                }

            } elseif ($exportwithzero === 1 AND $product['quantity'] <= 0) {
                $this->doDebug('<span style="color: grey;">' . $product['name'] . '</span> ustawiono stan mag. na 1');
                $product['quantity'] = 1;

            } elseif ($exportwithzero === 2 AND $product['quantity'] <= 0) {
                $this->doDebug('<span style="color: grey;">' . $product['name'] . '</span> ustawiono stan mag. na 0');
                $product['quantity'] = 0;

            }

            if ($reduceavailibilty === 1 AND $product['quantity'] > 1) {
                $this->doDebug('<span style="color: grey;">' . $product['name'] . '</span> Redukcja stanu magazynowego z ' . $product['quantity'] . ' na 1');
                $product['quantity'] = 1;
            }
        }
        return $Products;
    }

    /**
     * Get all attributes for product by product_id
     *
     * @param $product_id
     *
     * @return array|false|mysqli_result|null|PDOStatement|resource
     */
    public function getAttributesForProduct($product_id)
    {
        $attributes = Db::getInstance()->executeS($q = "
			SELECT DISTINCT GROUP_CONCAT(DISTINCT al.name SEPARATOR  ';') AS attribute_name, agl.name AS group_name, is_color_group
		    FROM " . _DB_PREFIX_ . "product_attribute
		    JOIN " . _DB_PREFIX_ . "product_attribute_combination USING (id_product_attribute)
			JOIN " . _DB_PREFIX_ . "attribute USING (id_attribute)
			JOIN " . _DB_PREFIX_ . "attribute_group_lang agl USING (id_attribute_group)
			JOIN " . _DB_PREFIX_ . "attribute_lang al USING (id_attribute)
			JOIN " . _DB_PREFIX_ . "attribute_group ag USING (id_attribute_group)
			WHERE agl.id_lang = " . (int)$this->getLangId() . "
			AND id_product = " . (int)$product_id . "
			GROUP BY group_name");

        return $attributes ? $attributes : array();
    }

    /**
     * Check if attribute exists in database
     *
     * @param $attribute_name
     *
     * @return bool
     */
    public function checkAttribute($attribute_name)
    {
        $exists = Db::getInstance()->executeS("
			SELECT 1 FROM " . _DB_PREFIX_ . "attribute_group_lang
			WHERE lower(name) = lower('" . $attribute_name . "')
			AND id_lang = " . (int)$this->getLangId() . "
			LIMIT 1");

        return $exists ? true : false;
    }

    /**
     * @param $string
     *
     * @return string
     */
    public static function htmlentities($string)
    {
        $string = str_replace('&', '&amp;', $string);

        return htmlspecialchars($string, ENT_COMPAT, 'UTF-8');
    }

    /**
     * Get the current version of okazjeintegrator module
     *
     * @return bool|string
     */
    public function getVersion()
    {
        if (isset($this->current_version)) {
            return $this->current_version;
        }

        $ctx = stream_context_create(array('http' => array('timeout' => 50)));
        $version = file_get_contents("http://www.okazje.info.pl/pdf/okazjeintegrator.version", 0, $ctx);

        return $version ? trim($version) : false;
    }
}
